'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _isUndefined = _interopDefault(require('lodash/isUndefined'));
var _isEmpty = _interopDefault(require('lodash/isEmpty'));
var _isPlainObject = _interopDefault(require('lodash/isPlainObject'));
var _defineProperty = _interopDefault(require('@babel/runtime/helpers/defineProperty'));
var _objectSpread = _interopDefault(require('@babel/runtime/helpers/objectSpread'));
var _toConsumableArray = _interopDefault(require('@babel/runtime/helpers/toConsumableArray'));
var _typeof = _interopDefault(require('@babel/runtime/helpers/typeof'));
var _slicedToArray = _interopDefault(require('@babel/runtime/helpers/slicedToArray'));
var axios = _interopDefault(require('axios'));

/**
 * 转换 text 中在 data 中出现的占位为最终字符串
 * 例如：text = 'Hello, {name}!', data = { name: 'Nicolas' } 会转换为 'Hello, Nicolas!'
 * 如果 dataReplaceable 设置为 true，会删除 data 中被 text 匹配到的值
 * @param {String} text 
 * @param {Object} data 
 * @param {Boolean} dataReplaceable 
 */
function parseTextPlaceholder(text, data) {
  var dataReplaceable = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
  var rPlaceholder = /\{([^}]+)\}/g;

  if (rPlaceholder.test(text) && _isPlainObject(data) && !_isEmpty(data)) {
    return text.replace(rPlaceholder, function (match, placeholder) {
      var val = data[placeholder];

      if (!_isUndefined(val)) {
        if (dataReplaceable) {
          delete data[placeholder];
        }

        return val;
      }

      return match;
    });
  }

  return text;
} // example: format('Do {0} love {1}? Yes, {2} love {0}!', 'you', 'me', 'I');
// return: 'Do you love me? Yes, I love You!'

function parseNumberPlaceholder(text) {
  for (var _len = arguments.length, params = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    params[_key - 1] = arguments[_key];
  }

  return text.replace(/\{(\d+)\}/g, function (m, i) {
    return params[i];
  });
}
function formatSize(bytes) {
  var i = -1;

  do {
    bytes = bytes / 1024;
    i++;
  } while (bytes >= 1024);

  return parseFloat(Math.max(bytes, 0.1).toFixed(2)) + ['KB', 'MB', 'GB', 'TB', 'PB', 'EB'][i];
}
function trim(text) {
  return text.replace(/^\s\s*/, '').replace(/\s\s*$/, '');
}
var html = {
  entityify: function entityify(text) {
    var character = {
      '<': '&lt;',
      '>': '&gt;',
      '&': '&amp;',
      '"': '&quot;'
    };
    return text.replace(/[<>"&]/g, function (match) {
      return character[match];
    });
  },
  deentityify: function deentityify(text) {
    var entity = {
      quot: '"',
      lt: '<',
      gt: '>'
    };
    return text.replace(/&([^&;]+);/g, function (match, key) {
      var ret = entity[key];
      return typeof ret === 'string' ? ret : match;
    });
  },
  strip: function strip(text) {
    return text.replace(/<(?:.|\s)*?>/g, '');
  },
  escape: function escape(text) {
    if (typeof text !== 'string') {
      return '';
    }

    var rscript = /<script[^>]*>(?:(?:(?!<\/script>).)*<\/script>)?/gi,
        rstyle = /<style[^>]*>(?:(?!@import|<\/style>).)*@import(?:(?!<\/style>).)+<\/style>/gi,
        rlink = /<link(?:(?!\.css).)+\.css[^>]*>/gi,
        rinnerevent = /on[a-zA-Z]+\s*=\s*(?:'[^']*'|"[^"]*"|[^\s\/>]+)/gi,
        rinnerexpress = /javascript:/gi;
    return text.replace(rscript, '').replace(rstyle, '').replace(rlink, '').replace(rinnerevent, '').replace(rinnerexpress, '');
  },
  filter: function filter(text, maxlength) {
    if (text === '') return '';
    text = text // 将2个以上的空字符转换为一个
    .replace(/\s{2,}/g, ' ') // 将所有HTML的换行标记转换为换行符
    .replace(/<br\s*\/?>/g, '\n') // 将所有HTML的空格标记转换为一个空字符
    .replace(/(\s*&(n|N)(b|B)(s|S)(p|P);\s*)+/g, ' ') // 将单个单引号转换为成对的单引号
    .replace(/'/g, "''"); // 过滤掉两端空格及HTML标记

    text = trim(text);
    text = this.strip(text);

    if (typeof maxlength === 'number') {
      text = text.substring(0, maxlength);
    }

    return text;
  }
};

var merge = function merge(left, right) {
  var ret = [];
  var l = 0,
      r = 0;

  while (l < left.length && r < right.length) {
    if (left[l] < right[r]) {
      ret.push(left[l++]);
    } else {
      ret.push(right[r++]);
    }
  }

  return ret.concat(_toConsumableArray(left.slice(l)), _toConsumableArray(right.slice(r)));
}; // 排序算法 - 并归排序
// 觉得挺有意思的，就拿来实现下


function mergeSort(data) {
  var len = data.length;

  if (len === 1) {
    return data;
  }

  var mid = Math.floor(len / 2),
      left = data.slice(0, mid),
      right = data.slice(mid);
  return merge(mergeSort(left), mergeSort(right));
}
function arrayToTree(data) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$id = _ref.id,
      id = _ref$id === void 0 ? 'id' : _ref$id,
      _ref$parentId = _ref.parentId,
      parentId = _ref$parentId === void 0 ? 'parentId' : _ref$parentId,
      _ref$rootParentId = _ref.rootParentId,
      rootParentId = _ref$rootParentId === void 0 ? 0 : _ref$rootParentId,
      _ref$children = _ref.children,
      children = _ref$children === void 0 ? 'children' : _ref$children;

  var list = [],
      map = {},
      tree = [];

  for (var i = 0; i < data.length; i++) {
    map[data[i][id]] = i;
    list.push(_objectSpread({}, data[i], _defineProperty({}, children, [])));
  }

  for (var _i = 0; _i < list.length; _i++) {
    var node = list[_i];

    if (node[parentId] !== rootParentId) {
      var foundIndex = map[node[parentId]];

      if (typeof foundIndex === 'number') {
        var found = list[foundIndex];
        found.children.push(node);
      }
    } else {
      tree.push(node);
    }
  }

  return tree;
} // 任务分割函数，当一个函数运行时间过长，可以使用该函数把它拆分为一系列能在较短时间内完成的子函数，使用定时器加入到队列。

function chunk(data, process, context) {
  var duration = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 100;
  var tasks = data.concat();

  var run = function run() {
    setTimeout(function () {
      process.call(context, tasks.shift());

      if (tasks.length) {
        run();
      }
    }, duration);
  };

  run();
} // 在一定时间（毫秒）内批量处理数据量比较大的数组，以减轻运行大数据数组对客户端的程序阻塞

function batch(data, process, context) {
  var cb = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : noop;

  var _ref2 = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {},
      _ref2$runDuration = _ref2.runDuration,
      runDuration = _ref2$runDuration === void 0 ? 25 : _ref2$runDuration,
      _ref2$chunkDuration = _ref2.chunkDuration,
      chunkDuration = _ref2$chunkDuration === void 0 ? 50 : _ref2$chunkDuration;

  var tasks = data.concat();

  var run = function run() {
    setTimeout(function () {
      var start = Date.now();

      do {
        process.call(context, tasks.shift());
      } while (tasks.length && Date.now() - start < chunkDuration);

      if (tasks.length) {
        run();
      } else {
        cb();
      }
    }, runDuration);
  };

  run();
}

function noop() {}

function type(obj) {
  if (obj == null) {
    return obj + '';
  }

  var types = 'Number String Boolean Array Function Object Math Date RegExp Error'.split(' ');
  var class2Type = {};
  types.forEach(function (type) {
    class2Type["[object ".concat(type, "]")] = type.toLowerCase();
  });
  return class2Type[Object.prototype.toString.call(obj)] || _typeof(obj);
}
/**
 * Deep merge objects.
 * @param {Object} target
 * @param {Object} sources
 */

function deepAssign(target) {
  for (var _len = arguments.length, sources = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    sources[_key - 1] = arguments[_key];
  }

  if (!sources.length) return target;
  var source = sources.shift();

  if ((type(target) === 'object' || type(target) === 'array') && (type(source) === 'object' || type(source) === 'array')) {
    var _arr = Object.keys(source);

    for (var _i = 0; _i < _arr.length; _i++) {
      var key = _arr[_i];

      if (type(source[key]) === 'object' || type(source[key]) === 'array') {
        if (!target[key]) {
          Object.assign(target, _defineProperty({}, key, type(source[key]) === 'object' ? {} : []));
        }

        deepAssign(target[key], source[key]);
      } else {
        // 如果 target 是数组，会被 Object.assign 视为属性名为 0、1、2... 的对象
        // { [prop]: source[prop] } -> { 0: 'a' } 会覆盖 target 的 index 为 0 的值
        Object.assign(target, _defineProperty({}, key, source[key]));
      }
    }
  }

  return deepAssign.apply(void 0, [target].concat(sources));
} // 返回介于 a 到 b 之间的一个随机数

function random(a, b) {
  var val = b - a + 1;
  return Math.floor(Math.random() * val + a);
}
function delayTask(task) {
  var delay = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 600;
  var running = false;
  var taskTimer = setTimeout(function () {
    running = true;
    task();
  }, delay); // return clear state, true: cleared, false: not cleared

  return function () {
    clearTimeout(taskTimer);
    return !running;
  };
}

/**
 * 
 * TODO: 该模块下方法还未做过任何测试
 */
function isLeapYear(year) {
  return year % 4 === 0 && (year % 100 !== 0 || year % 400 === 0);
}
function parseDate(format, value) {
  if (value === '') {
    return null;
  }

  var year = -1,
      month = -1,
      day = -1,
      iValue = 0,
      date = new Date(),
      iFormat,
      extra;

  var lookAhead = function lookAhead(match) {
    var matches = iFormat + 1 < format.length && format.charAt(iFormat + 1) === match;

    if (matches) {
      iFormat++;
    }

    return matches;
  },
      getNumber = function getNumber(match) {
    var isDoubled = lookAhead(match),
        size = match === 'y' && isDoubled ? 4 : 2,
        digits = new RegExp('^\\d{1,' + size + '}'),
        num = value.substring(iValue).match(digits);

    if (!num) {
      throw 'Missing number at position ' + iValue;
    }

    iValue += num[0].length;
    return parseInt(num[0], 10);
  },
      checkLiteral = function checkLiteral() {
    if (value.charAt(iValue) !== format.charAt(iFormat)) {
      throw 'Unexpected literal at position ' + iValue;
    }

    iValue++;
  };

  for (iFormat = 0; iFormat < format.length; iFormat++) {
    switch (format.charAt(iFormat)) {
      case 'd':
        day = getNumber('d');
        break;

      case 'm':
        month = getNumber('m');
        break;

      case 'y':
        year = getNumber('y');
        break;

      default:
        checkLiteral();
    }
  }

  if (iValue < value.length) {
    extra = value.substr(iValue);

    if (!/^\s+/.test(extra)) {
      throw 'Extra/unparsed characters found in date: ' + extra;
    }
  }

  if (year === -1) {
    year = date.getFullYear();
  } else if (year < 100) {
    year += date.getFullYear() - date.getFullYear() % 100;
  }

  date = new Date(year, month - 1, day);

  if (date.getFullYear() !== year || date.getMonth() + 1 !== month || date.getDate() !== day) {
    throw 'Invalid date';
  }

  return date;
}
function formatDate(format, date) {
  if (!date) {
    return '';
  }

  var output = '',
      iFormat,
      lookAhead = function lookAhead(match) {
    var matches = iFormat + 1 < format.length && format.charAt(iFormat + 1) === match;

    if (matches) {
      iFormat++;
    }

    return matches;
  },
      formatNumber = function formatNumber(match, value, len) {
    var num = '' + value;

    if (lookAhead(match)) {
      while (num.length < len) {
        num = '0' + num;
      }
    }

    return num;
  };

  for (iFormat = 0; iFormat < format.length; iFormat++) {
    switch (format.charAt(iFormat)) {
      case 'd':
        output += formatNumber('d', date.getDate(), 2);
        break;

      case 'm':
        output += formatNumber('m', date.getMonth() + 1, 2);
        break;

      case 'y':
        output += lookAhead('y') ? date.getFullYear() : (date.getYear() % 100 < 10 ? '0' : '') + date.getYear() % 100;
        break;

      default:
        output += format.charAt(iFormat);
    }
  }

  return output;
}

/**
 * 将提供的毫秒值解析成格式化的时间数值列表
 * @param {Object} options { startUnit, labels }
 *  startUnit: 'years | months | weeks | days | hours | minutes | seconds'
 *  labels: { years: '年', months: '月', ... }
 * 
 * @returns (time) => [{ unit: 'Years', value }, ...]
 */

function timeParser() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var _options$startUnit = options.startUnit,
      startUnit = _options$startUnit === void 0 ? 'years' : _options$startUnit,
      _options$units = options.units,
      units = _options$units === void 0 ? {} : _options$units;
  var unitsSource = [{
    unit: 'Years',
    value: 12
  }, {
    unit: 'Months',
    value: 4
  }, {
    unit: 'Weeks',
    value: 7
  }, {
    unit: 'Days',
    value: 24
  }, {
    unit: 'Hours',
    value: 60
  }, {
    unit: 'Minutes',
    value: 60
  }, {
    unit: 'Seconds',
    value: 1000
  }];

  var inAll = function inAll(arr, option) {
    return arr.reduce(function (prev, cur) {
      if (option === 'add') {
        return prev + cur;
      }

      return prev * cur;
    });
  };

  var getSourceValues = function getSourceValues(units) {
    return units.map(function (unit) {
      return unit.value;
    });
  };

  unitsSource = unitsSource.map(function (source, i) {
    var key = source.unit.toLowerCase(),
        ms = inAll(getSourceValues(unitsSource.slice(i)));
    return _objectSpread({}, source, {
      unit: units[key] || source.unit,
      key: key,
      ms: ms
    });
  });
  var startIndex = unitsSource.findIndex(function (source) {
    return source.key === startUnit;
  });

  if (startIndex > 0) {
    unitsSource = unitsSource.slice(startIndex);
  }

  var cache = {};
  return function (time) {
    if (type(time) !== 'number' || time < 0) {
      return unitsSource.map(function (_ref) {
        var unit = _ref.unit;
        return {
          value: 0,
          unit: unit
        };
      });
    }

    var values = [];
    unitsSource.reduce(function (prev, source, i) {
      var value = Math.floor(time / unitsSource[i].ms) - prev;
      values.push({
        value: value,
        unit: source.unit
      });
      prev = values.map(function (n, j) {
        var unitValues = cache["".concat(j, "-").concat(i + 1)];

        if (!unitValues) {
          unitValues = inAll(getSourceValues(unitsSource.slice(j, i + 1)));
          cache["".concat(j, "-").concat(i + 1)] = unitValues;
        }

        return n.value * unitValues;
      });
      prev = inAll(prev, 'add');
      return prev;
    }, 0);
    return values;
  };
}
/**
 * 秒钟倒计时工具，返回 stop 方法，执行可用来停止 tick 执行
 * @param {Number} second 
 * @param {Object} events: { onStart, onProgress, onEnd } 
 * @param {Object} context 
 */

function countdown(value) {
  var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref2$onStart = _ref2.onStart,
      onStart = _ref2$onStart === void 0 ? noop$1 : _ref2$onStart,
      _ref2$onProgress = _ref2.onProgress,
      onProgress = _ref2$onProgress === void 0 ? noop$1 : _ref2$onProgress,
      _ref2$onEnd = _ref2.onEnd,
      onEnd = _ref2$onEnd === void 0 ? noop$1 : _ref2$onEnd;

  var context = arguments.length > 2 ? arguments[2] : undefined;

  if (type(value) !== 'date' && type(value) !== 'number') {
    throw new Error('The value of params must be a Date or Number.');
  }

  var cdType = type(value) === 'date' ? 'date' : 'second';
  var last = value;
  var times = timeParser({
    startUnit: 'hours'
  });
  var timerId;

  var tick = function tick() {
    timerId = setTimeout(function () {
      if (cdType === 'date') {
        var dif = last - Date.now(); // 缓存 value 变量，方便在执行停止时可以传入 stop

        value = times(dif);

        if (dif <= 0) {
          return onEnd.bind(context)(value);
        }

        onProgress.bind(context)(value);
      } else {
        if (!value) {
          return onEnd.bind(context)(value);
        }

        onProgress.bind(context)(--value);
      }

      tick();
    }, 1000);
  };

  return {
    start: function start() {
      if (cdType === 'date') {
        value = times(last - Date.now());
      }

      onStart.bind(context)(value);
      tick();
    },
    stop: function stop(callback) {
      clearTimeout(timerId);
      callback && callback.bind(context)(value);
    }
  };
}
function noop$1() {}

/**
 * 对 localStorage 的高级封装
 * 与 localStorage 不同的是，set 方法的 value 可以是任何类型的值，通过 get 方法返回之前存入的值，返回值根据存入的类型而定
 */

var cache = {
  get: function get(key) {
    return JSON.parse(localStorage.getItem(key));
  },
  set: function set(key, value) {
    value = JSON.stringify(value); // 在有些移动端机型，同时执行 localStorage.setItem 多次会导致异常

    try {
      localStorage.setItem(key, value);
    } catch (ex) {
      console.error("cache.set(".concat(key, ", ").concat(value, ") error: ").concat(ex.message, "."));
    }
  },
  remove: function remove(key) {
    localStorage.removeItem(key);
  },
  clear: function clear() {
    localStorage.clear();
  }
};
/**
 * 把 cache 作为一个数据表一样管理
 */

var cacheTable = {
  get: function get(primaryKey, secondaryKey) {
    var table = cache.get(primaryKey);

    if (type(table) !== 'object' || type(table[secondaryKey]) === 'undefined') {
      return null;
    }

    return table[secondaryKey];
  },
  // 如果原有数据和 value 都是普通对象，那么 value 将与原有数据合并
  // 否则 value 覆盖原有数据
  set: function set(primaryKey, secondaryKey, value) {
    var table = cache.get(primaryKey) || {},
        row = table[secondaryKey];

    if (row && type(row) === type(value) && _isPlainObject(value)) {
      table[secondaryKey] = Object.assign({}, row, value);
    } else {
      table[secondaryKey] = value;
    }

    cache.set(primaryKey, table);
  },
  remove: function remove(primaryKey, secondaryKey) {
    var table = cache.get(primaryKey);

    if (type(table) !== 'object') {
      return;
    }

    delete table[secondaryKey];
    cache.set(primaryKey, table);
  },
  clear: function clear(primaryKey) {
    var table = cache.get(primaryKey);

    if (type(table) !== 'object') {
      return;
    }

    cache.set(primaryKey, null);
  }
};
var cookie = {
  get: function get(name) {
    var pattern = "(?:;\\s)?".concat(encodeURIComponent(name), "=([^;]*);?"),
        rCookie = new RegExp(pattern);
    return rCookie.test(document.cookie) ? decodeURIComponent(RegExp['$1']) : null;
  },
  set: function set(name, value) {
    var _ref = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
        expires = _ref.expires,
        path = _ref.path,
        domain = _ref.domain,
        secure = _ref.secure;

    var cookieText = "".concat(encodeURIComponent(name), "=").concat(encodeURIComponent(value));

    if (expires instanceof Date) {
      cookieText += '; expires=' + expires.toGMTString();
    }

    if (path) {
      cookieText += '; path=' + path;
    }

    if (domain) {
      cookieText += '; domain=' + domain;
    }

    if (secure) {
      cookieText += '; secure';
    }

    document.cookie = cookieText;
  },
  remove: function remove(name) {
    var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
        path = _ref2.path,
        domain = _ref2.domain,
        secure = _ref2.secure;

    var options = {
      // 设置失效时间为1970年1月1日（初始化为0ms的Date对象的值）
      expires: new Date(0),
      path: path,
      domain: domain,
      secure: secure
    };
    this.set(name, '', options);
  }
};
/**
 * 监听页面可见性变化并绑定处理函数
 * HTML 5 Page Visibility API: https://developer.mozilla.org/en-US/docs/Web/API/Page_Visibility_API
 * 
 * 
 * 基本用法：
 * listenPageVisibility(visible => {
 *   if (visible) {
 *     // 页面从不可见变为可见
 *   } else {
 *     // 页面从可见变为不可见
 *   }
 * });
 * 
 * 移动端兼容性：
 * iOS:             √ 自带浏览器    √ Chrome for iOS        × UC 浏览器    √ 微信    × QQ    × 微博    × 沪江 Web 容器（支持自身native页面切换）
 * Android 4.4+:    √ 自带浏览器    √ Chrome for Android    √ UC 浏览器    √ 微信    √ QQ    √ 微博    √ 沪江 Web 容器
 * 
 */

function listenPageVisibility(handler) {
  var hidden, eventType;

  if (typeof document.hidden !== 'undefined') {
    hidden = 'hidden';
    eventType = 'visibilitychange';
  } else if (typeof document.webkitHidden !== 'undefined') {
    hidden = 'webkitHidden';
    eventType = 'webkitvisibilitychange';
  }

  document.addEventListener(eventType, function () {
    if (typeof handler === 'function') {
      handler(!document[hidden]);
    }
  }, false);
} // source: http://stackoverflow.com/questions/123999/how-to-tell-if-a-dom-element-is-visible-in-the-current-viewport/7557433#7557433

function isElementInViewport(el) {
  var rect = el.getBoundingClientRect();
  return rect.top >= 0 && rect.left >= 0 && rect.bottom <= (window.innerHeight || document.documentElement.clientHeight) && rect.right <= (window.innerWidth || document.documentElement.clientWidth);
} // TODO: 还未经过测试

function lazyLoadImage() {
  var dataSrcAttr = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'data-src';
  var container = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : document;
  var eventTypes = 'DOMContentLoaded load resize scroll'.split(' '),
      images = container.querySelectorAll("img[".concat(dataSrcAttr, "]"));

  if (!images.length) {
    return;
  }

  var loadImage = function loadImage() {
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = images[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var image = _step.value;
        var src = void 0;

        if (isElementInViewport(image) && (src = image.getAttribute(dataSrcAttr))) {
          image.setAttribute('src', src);
          image.removeAttribute(dataSrcAttr);
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return != null) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }
  };

  eventTypes.forEach(function (type$$1) {
    window.addEventListener(type$$1, loadImage, false);
  });
}
function loadFile(type$$1, url, callback, context) {
  var el;

  if (type$$1 === 'js') {
    el = document.createElement('script');
    el.type = 'text/javascript';
  } else if (type$$1 === 'css') {
    el = document.createElement('link');
    el.rel = 'stylesheet';
    el.type = 'text/css';
  }

  if (callback) {
    // IE
    if (el.readyState) {
      el.onreadystatechange = function () {
        if (el.readyState == 'loaded' || el.readyState == 'complete') {
          // 防止其他 ready 状态更改时再次触发
          el.onreadystatechange = null;
          callback.bind(context)();
        }
      };
    } else {
      el.onload = function () {
        callback.bind(context)();
      };
    }
  }

  el[type$$1 === 'js' ? 'src' : 'href'] = url;
  document.getElementsByTagName('head')[0].appendChild(el);
}

var rUrl = /^([^?#]+)(\?[^#]+)?(#.+)?/;
function getQuerys() {
  var url = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  var rQueryValue = /^([^=]+)(?:=(.*))?$/;

  var _ref = rUrl.exec(url) || [],
      _ref2 = _slicedToArray(_ref, 3),
      search = _ref2[2];

  if (!search) {
    return null;
  }

  var ret = {},
      args = search.substring(1).split('&'),
      len = args.length;

  for (var i = 0; i < len; i++) {
    var queryMatch = rQueryValue.exec(args[i]);

    if (queryMatch) {
      var _queryMatch = _slicedToArray(queryMatch, 3),
          name = _queryMatch[1],
          value = _queryMatch[2];

      name = decodeURIComponent(name);
      value = typeof value !== 'undefined' ? decodeURIComponent(value) : '';

      if (/^(true|false)$/.test(value)) {
        value = JSON.parse(RegExp.$1);
      } else if (/^\d+$/.test(value)) {
        value = +value;
      }

      ret[name] = value;
    }
  }

  return ret;
}
function addQuerys() {
  var url = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  var querys = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var oriQuerys = getQuerys(url) || {};

  var _arr = Object.keys(querys);

  for (var _i = 0; _i < _arr.length; _i++) {
    var name = _arr[_i];
    oriQuerys[name] = querys[name];
  }

  var queryString = [];

  var _arr2 = Object.keys(oriQuerys);

  for (var _i2 = 0; _i2 < _arr2.length; _i2++) {
    var _name = _arr2[_i2];
    var key = encodeURIComponent(_name),
        value = typeof oriQuerys[_name] !== 'undefined' ? encodeURIComponent(oriQuerys[_name]) : '';
    queryString.push("".concat(key, "=").concat(value));
  }

  if (!queryString.length) {
    return url;
  }

  var _rUrl$exec = rUrl.exec(url),
      _rUrl$exec2 = _slicedToArray(_rUrl$exec, 4),
      baseUrl = _rUrl$exec2[1],
      _rUrl$exec2$ = _rUrl$exec2[3],
      hash = _rUrl$exec2$ === void 0 ? '' : _rUrl$exec2$;

  return "".concat(baseUrl, "?").concat(queryString.join('&')).concat(hash);
}
/**
 * 参数可通过多种方式传入
 * querys(); -> { ...querys }
 * querys(url); -> { ...querys }
 * querys(params); -> 'url?params#hash'
 * querys(url, params); -> 'url?params#hash'
 * 
 * @param {String} url
 * @param {Object} params: 需要添加到 URL search 部分的参数，如果传了该参数，会调用 addQuerys 方法
 */

function querys() {
  for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  var url = args[0],
      params = args[1];

  if (!url) {
    url = window.location.href;
  } else if (_typeof(url) === 'object') {
    var _ref3 = [window.location.href, url];
    url = _ref3[0];
    params = _ref3[1];
  }

  if (!params) {
    return getQuerys(url);
  }

  return addQuerys(url, params);
}
function checkQuery() {
  var requiredParams = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var query = querys() || {};
  var ret = requiredParams.every(function (param) {
    return query[param] || query[param.toLowerCase()];
  });

  if (!ret) {
    return Promise.reject(new Error("Missing url params: ".concat(requiredParams.join(', '))));
  }

  return Promise.resolve(query);
}

var message = '请求接口"{url}"时出错，错误信息：{message}';

var request = function request(url, options) {
  var requestPromise = fetch(url, options).then(function (res) {
    return res.json().then(function (json) {
      if (json.status === 0) {
        return json.data;
      } else {
        return Promise.reject(json);
      }
    }, function (err) {
      throw new Error(parseTextPlaceholder(message, {
        url: url,
        message: err.message
      }));
    });
  }, function (err) {
    throw new Error(parseTextPlaceholder(message, {
      url: url,
      message: err.message
    }));
  });
  var abort;
  var abortPromise = new Promise(function (resolve, reject) {
    abort = function abort() {
      reject(new Error(parseTextPlaceholder(message, {
        url: url,
        message: 'abort'
      })));
    };
  });
  var tasks = [requestPromise, abortPromise],
      timeout = options.timeout;
  var timeoutPromise;

  if (timeout) {
    delete options.timeout;
    timeoutPromise = new Promise(function (resolve, reject) {
      setTimeout(function () {
        return reject(new Error(parseTextPlaceholder(message, {
          url: url,
          message: '请求超时'
        })));
      }, timeout);
    });
    tasks.push(timeoutPromise);
  }

  var p = Promise.race(tasks);
  p.abort = abort;
  return p;
};

var types = 'get post put patch delete'.split(' ');
types.forEach(function (type) {
  /**
   * 异步数据请求 window.fetch 的高级封装
   * @param url: (String) url字符串中可以包含占位符(RUSTful API)，用传入的data参数，解析这些占位符
   *   例如：
   *   url = '//xx.xxx.com/api/user/{userId}', data = {userId: 123, userName: 'Nicolas'}
   *   --> url: '//xx.xxx.com/api/user/123'
   *   --> data: {userName: 'Nicolas'}
   *
   * @param data: (Object) 如果是get和delete请求，该对象会被追加到url的query部分。否则，该对象会JSON.stringify，
   * 然后配置到请求设置项的body中。注意：data中如果匹配到url中的占位字符，相应的数据在解析完成后会从data中移除
   *
   * @param options (Object) fetch API 的设置选项，另外增加了额外的timeout(Number)参数
   */
  request[type] = function (url, data, options) {
    url = parseTextPlaceholder(url, data, true);
    options = Object.assign({
      method: type.toUpperCase()
    }, request.defaults, options);

    if (type === 'get' || type === 'delete') {
      if (_isPlainObject(data) && !_isEmpty(data)) {
        url = querys(url, data);
      }
    } else {
      options = Object.assign(options, {
        body: _isPlainObject(data) && !_isEmpty(data) ? JSON.stringify(data) : ''
      });
    }

    return request(url, options);
  };
}); // 可以设置通用的默认配置项来统一处理请求设置项

request.defaults = {};

function request$1() {
  var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      filterResponse = _ref.filterResponse,
      beautifyError = _ref.beautifyError;

  var inst = axios.create(config);
  return 'get delete head options post put patch'.split(' ').reduce(function (req, method) {
    req[method] = function (url, data, config) {
      var methodsWithBody = ['post', 'put', 'patch'];
      var xhr = null;
      url = parseTextPlaceholder(url, data, true);

      if (methodsWithBody.includes(method)) {
        xhr = inst[method](url, data, config);
      } else {
        xhr = inst[method](url, _objectSpread({
          params: data
        }, config));
      }

      return xhr.then(function (_ref2) {
        var data = _ref2.data;

        if (typeof filterResponse === 'function') {
          return filterResponse(data);
        }

        return data;
      }).catch(function (err) {
        if (typeof beautifyError === 'function') {
          return Promise.reject(beautifyError(url, err));
        }

        return Promise.reject(err);
      });
    };

    return req;
  }, {
    inst: inst
  });
}

/*
 * url: string
 * settings: object
 *   jsonpCallback: string
 *   timeout: number
 *   done: function
 *   fail: function
 */
function jsonp(url, settings) {
  var script = document.createElement('script');
  var obj, props, jsonpCallback, response, i, timer;
  script.type = 'text/javascript';
  jsonpCallback = settings.jsonpCallback;

  if (jsonpCallback.indexOf('.') !== -1) {
    props = jsonpCallback.split('.');
    window[props[0]] = obj = {};

    for (i = 1; i < props.length; i++) {
      if (i < props.length - 1) {
        obj = obj[props[i]] = {};
      } else {
        obj[props[i]] = function () {
          response = arguments[0];
        };
      }
    }
  } else {
    window[jsonpCallback] = function () {
      response = arguments[0];
    };
  }

  if (settings.timeout) {
    timer = setTimeout(function () {
      delete settings.done;

      if (typeof settings.fail === 'function') {
        settings.fail();
      }
    }, settings.timeout);
  }

  if (script.readyState) {
    script.onreadystatechange = function () {
      if (script.readyState === 'loaded' || script.readyState === 'complete') {
        script.onreadystatechange = null;
        clearTimeout(timer);

        if (typeof settings.done === 'function') {
          settings.done(response);
        }
      }
    };
  } else {
    script.onload = function () {
      clearTimeout(timer);

      if (typeof settings.done === 'function') {
        settings.done(response);
      }
    };
  }

  script.src = url;
  (document.head || document.getElementsByTagName('head')[0]).appendChild(script);
}

var History = function History() {
  this.stack = [];
  this.activeIndex = 0;
};

History.prototype = {
  constructor: History,
  getActive: function getActive() {
    return this.stack[this.activeIndex];
  },
  getPrev: function getPrev() {
    return this.stack[this.activeIndex - 1];
  },
  getNext: function getNext() {
    return this.stack[this.activeIndex + 1];
  },
  size: function size() {
    return this.stack.length;
  },
  add: function add(url) {
    if (this.getNext()) {
      this.stack = this.stack.slice(0, this.activeIndex + 1);
    }

    this.stack.push(url);
    this.activeIndex = this.stack.length - 1;
    return this.stack.length;
  },
  find: function find(url) {
    var length = this.stack.length,
        index,
        i;

    for (i = 0; i < length; i++) {
      if (url === this.stack[i]) {
        index = i;
        break;
      }
    }

    return index;
  },
  direct: function direct(url) {
    var newActiveIndex = this.find(url),
        a = this.activeIndex;

    if (typeof newActiveIndex !== 'undefined') {
      this.activeIndex = newActiveIndex;
    }

    if (newActiveIndex < a) {
      return -1;
    } else if (newActiveIndex > a) {
      return 1;
    } else {
      return this.add(url);
    }
  }
};

exports.fetchRequest = request;
exports.axiosRequest = request$1;
exports.jsonp = jsonp;
exports.History = History;
exports.parseTextPlaceholder = parseTextPlaceholder;
exports.parseNumberPlaceholder = parseNumberPlaceholder;
exports.formatSize = formatSize;
exports.trim = trim;
exports.html = html;
exports.mergeSort = mergeSort;
exports.arrayToTree = arrayToTree;
exports.chunk = chunk;
exports.batch = batch;
exports.type = type;
exports.deepAssign = deepAssign;
exports.random = random;
exports.delayTask = delayTask;
exports.isLeapYear = isLeapYear;
exports.parseDate = parseDate;
exports.formatDate = formatDate;
exports.timeParser = timeParser;
exports.countdown = countdown;
exports.noop = noop$1;
exports.cache = cache;
exports.cacheTable = cacheTable;
exports.cookie = cookie;
exports.listenPageVisibility = listenPageVisibility;
exports.isElementInViewport = isElementInViewport;
exports.lazyLoadImage = lazyLoadImage;
exports.loadFile = loadFile;
exports.getQuerys = getQuerys;
exports.addQuerys = addQuerys;
exports.querys = querys;
exports.checkQuery = checkQuery;
